<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Price extends \Zeedhi\Framework\Controller\Simple {

    protected $priceService;
    protected $util;
    protected $snackMessage;

    public function __construct(\Service\PriceService $priceService, \Util\SnackMessage $snackMessage, \Util\Util $util){
        $this->priceService = $priceService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
    }

    public function getPrice($productCode, $CDCLIENTE, $CDFILIAL, $CDLOJA){
        try {
            $pricingDetails = array();

            $pricingTable = $this->getPricingTable($CDCLIENTE, $CDFILIAL, $CDLOJA);
            $effectiveDate = $this->checkEffective($pricingTable['CDFILTABPREC'], $pricingTable['CDTABEPREC']);

            $itemPrice = $this->checkPrice($pricingTable['CDFILTABPREC'], $pricingTable['CDTABEPREC'], $productCode, $effectiveDate);
            if (!Empty($itemPrice)){
                
                $return['PRECOVAR']   = floatval($itemPrice['IDPRECVARIA']);
                $return['PRECOCLIE']  = floatval($itemPrice['PRECOCLIE']);

                $dailyPrice = $this->dailyPrice($pricingTable['CDFILTABPREC'], $pricingTable['CDTABEPREC'], $productCode, $effectiveDate);
                if (Empty($dailyPrice)){
                    $return['PRECOSUGER'] = floatval($itemPrice['PRECOSUGER']);
                    $return['PRECO'] = floatval($itemPrice['PRECO']);
                    $return['ACRE'] = 0;
                    $return['DESC'] = 0;
                }
                else {
                    $return["PRECOSUGER"] = 0;
                    if ($dailyPrice['PERC'] === 'V') $mod = $dailyPrice['VRPRECODIA'];
                    else $mod = round(($itemPrice['PRECO'] * $dailyPrice['VRPRECODIA']) / 100, 2);

                    if ($dailyPrice['DESCACRE'] === 'A'){
                        $return['DESC'] = 0;
                        if ($dailyPrice['IDVISUACUPOM'] === 'N'){
                            $return['ACRE'] = 0;
                            $return['PRECO'] = floatval(round($itemPrice['PRECO'] + $mod, 2));
                        }
                        else {
                            $return['ACRE'] = floatval($mod);
                            $return['PRECO'] = floatval(round($itemPrice['PRECO'], 2));
                        }
                    }
                    else {
                        $return['ACRE'] = 0;
                        if ($dailyPrice['IDVISUACUPOM'] === 'N'){
                            $return['DESC'] = 0;
                            $return['PRECO'] = floatval(round($itemPrice['PRECO'] - $mod, 2));
                        }
                        else {
                            $return['DESC'] = floatval($mod);
                            $return['PRECO'] = floatval(round($itemPrice['PRECO'], 2));
                        }
                    }
                }
            }

            return $return;

        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getPrices($CDFILIAL, $CDTABEPREC, $DTINIVGPREC){
        try {
            $prices = $this->priceService->getPrices($CDFILIAL, $CDTABEPREC, $DTINIVGPREC);
            if (!Empty($prices)){
                $indexedPrices = array();
                foreach ($prices as $priceItem){
                    $indexedPrices[$priceItem['CDPRODUTO']] = $priceItem;
                }
                return $indexedPrices;
            }
            else {
                throw new \UnexpectedValueException('Não existem precos cadastrados.');
            }
        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($this->snackMessage->getMessage('0').$e->getMessage());
        }
    }

    public function getPricingTable($CDCLIENTE, $CDFILIAL, $CDLOJA){
        try {
            $pricingTable = $this->priceService->clientBranchPricingTable($CDCLIENTE, $CDFILIAL);
            if (Empty($pricingTable)){
                $pricingTable = $this->priceService->clientPricingTable($CDCLIENTE);
                if (Empty($pricingTable)){
                    $pricingTable = $this->priceService->storePricingTable($CDFILIAL, $CDLOJA);
                    if (Empty($pricingTable)){
                        $pricingTable = $this->priceService->paravendPricingTable($CDFILIAL);
                    }
                }
            }
            if (!Empty($pricingTable)){
                if ($pricingTable['CDFILTABPREC'] === null){
                    $pricingTable['CDFILTABPREC'] = $CDFILIAL;
                }
            }
            else {
                throw new \UnexpectedValueException('Não há tabela de preços cadastrada.');
            }

            return $pricingTable;

        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($this->snackMessage->getMessage('0').$e->getMessage());
        }
    }

    public function checkEffective($CDFILIAL, $priceTable){
        try {
            $salesTable = $this->priceService->getSalesTable($CDFILIAL, $priceTable);
            if (!Empty($salesTable)) return $salesTable['DTINIVGPREC'];
            else throw new \UnexpectedValueException('Tabela de preços está inativa.');

        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($this->snackMessage->getMessage('0').$e->getMessage());
        }
    }

    public function checkPrice($CDFILIAL, $pricingTable, $productCode, $effectiveDate){
        try {
            $itemPrice = $this->priceService->getItemPrice($CDFILIAL, $pricingTable, $effectiveDate, $productCode);
            if (Empty($itemPrice)){
                throw new \UnexpectedValueException('Preço não cadastrado.');
            }
            else if ($itemPrice['PRECO']+$itemPrice['PRECOCLIE'] === 0){
                throw new \UnexpectedValueException('Produto sem preço cadastrado.');
            }
            else {
                return $itemPrice;
            }
        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($this->snackMessage->getMessage('0').$e->getMessage());
        }
    }

    public function dailyPrice($CDFILIAL, $pricingTableCode, $productCode, $effectiveDate){
        try {
            $currentDay = date('d/m/Y');
            $nextDay = date('d/m/Y', strtotime('+1 day'));

            if ($this->isHoliday($CDFILIAL, $currentDay))
                $day = 'F';
            else if ($this->isHoliday($CDFILIAL, $nextDay))
                $day = 'V';
            else
                $day = (int)date('w') + 1;
            
            $hour = date('Hi');
            
            $priceByDay = $this->priceService->getDailyPrice($CDFILIAL, $pricingTableCode, $effectiveDate, $productCode, $day, $hour);
            if (Empty($priceByDay)){
                $priceByDay = $this->priceService->getDailyPrice($CDFILIAL, $pricingTableCode, $effectiveDate, $productCode, 'T', $hour);
            }
            
            return $priceByDay;

        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }

    }

    public function isHoliday($CDFILIAL, $currentDay){
        try {
            $check = $this->priceService->holidayCheck($CDFILIAL, $currentDay);
            if (!Empty($check)) return true;
            return false;
        } catch (\Exception $e) {
            throw new \Exception($this->snackMessage->getMessage('0').$e->getMessage());
        }
    }

}