<?php
namespace Zeedhi\PaymentPlatform\Implementations\MundiPaggPHP;

use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyPlatform;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;

use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyPlatform;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;

use Zeedhi\PaymentPlatform\Consult\ConsultPlatform;
use Zeedhi\PaymentPlatform\Consult\Consult;

use Zeedhi\PaymentPlatform\Revoke\RevokePlatform;
use Zeedhi\PaymentPlatform\Revoke\Revoke;

use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Phone;
use Zeedhi\PaymentPlatform\RequestURL;

use Zeedhi\PaymentPlatform\Exception;



class PaymentImpl implements InstantBuyPlatform, NonInstantBuyPlatform, ConsultPlatform, RevokePlatform {
    /** @var String */
    private $merchantKey;
    /** @var HttpRequest */
    private $httpRequest;
    /** @var String */
    private $secretKey;
    /** @var String */
    private $basicAuthPassword;
    /** @var \MundiAPILib\MundiAPIClient */
    private $apiClient;

    /**
     * PaymentImpl constructor.
     *
     * @param String        $merchantKey
     * @param HttpRequest   $httpRequest
     * @param String        $basicAuthPassword
     */
    public function __construct($merchantKey, HttpRequest $httpRequest, $basicAuthPassword) {
        $this->merchantKey = $merchantKey;
        $this->httpRequest = $httpRequest;
        $this->requestHeaders = array(
            'Authorization: Basic '. base64_encode($this->merchantKey . ':'),
            'Content-Type:application/json'
        );
        $this->basicAuthPassword = $basicAuthPassword;
        $this->apiClient = new \MundiAPILib\MundiAPIClient(
            $merchantKey,
            $basicAuthPassword
        );
        global $apiClient;
    }
    
    private function configCustomer(Customer $customer) {
        $customerController = $this->apiClient->getCustomers();

        $request = new \MundiAPILib\Models\CreateCustomerRequest();
        $request->id = $customer->getId();
        $request->name = $customer->getName();
        $request->email = $customer->getEmail();
        $request->type = $customer->getType();
        $request->document = $customer->getDocument();
        $request->code = $customer->getCode();
        
        $request->address = new \MundiAPILib\Models\CreateAddressRequest();
        $request->address->line1 = $customer->getAddress()->getNumber().', '.$customer->getAddress()->getStreet().', '.$customer->getAddress()->getNeighborhood();
        $request->address->line2 = $customer->getAddress()->getComplement();
        $request->address->zipCode = $customer->getAddress()->getZipCode();
        $request->address->city = $customer->getAddress()->getCity();
        $request->address->state = $customer->getAddress()->getState();
        $request->address->country = $customer->getAddress()->getCountry();
        /*$request->address->metadata = new \MundiAPILib\Models\UpdateMetadataRequest();
        $request->address->metadata->id = $customer->getAddress()->getMetadata();*/

        $request->phones = new \MundiAPILib\Models\CreatePhonesRequest();
        $request->phones->homePhone = new \MundiAPILib\Models\CreatePhoneRequest();
        $request->phones->homePhone->number = $customer->getPhone()->getNumber();
        $request->phones->homePhone->areaCode = $customer->getPhone()->getAreaCode();
        $request->phones->homePhone->countryCode = $customer->getPhone()->getCountryCode();
        
        $result = $customerController->createCustomer($request);
        
        return $result;
    }
    
    public function createToken(CreditCard $card, Customer $customer) {
        $customerController = $this->apiClient->getCustomers();
        
        $request = $this->configCustomer($customer);
        $customerId = $request->id;
        
        $request = new \MundiAPILib\Models\CreateCardRequest();
        $request->number = $card->getNumbers();
        $request->holderName = $card->getHolderName();
        $request->expMonth = $card->getExpDateMonth();
        $request->expYear = $card->getExpDateYear();
        $request->cvv = $card->getCvv();
        $request->options = $card->getOptions();
        $request->privateLabel = $card->getPrivateLabel();
        $request->type = $card->getType();
        $request->metadata = $card->getMetadata();
        $request->brand = $card->getFlag();
        $request->holderDocument = $card->getHolderDocument();
        $request->privateLabel = $card->getPrivateLabel();
        
        $request->billingAddress = new \MundiAPILib\Models\CreateAddressRequest();
        $request->billingAddress->line1 = $card->getBillingAddress()->getNumber().', '.$card->getBillingAddress()->getStreet().', '.$card->getBillingAddress()->getNeighborhood();
        $request->billingAddress->line2 = $card->getBillingAddress()->getComplement();
        $request->billingAddress->city = $card->getBillingAddress()->getCity();
        $request->billingAddress->state = $card->getBillingAddress()->getState();
        $request->billingAddress->country = $card->getBillingAddress()->getCountry();
        $request->billingAddress->zipCode = $card->getBillingAddress()->getZipCode();
        $request->billingAddressId = $card->getBillingAddressId();
        
        $result = $customerController->createCard($customerId, $request);
        
        return array(
            "InstantBuyKey" => $result->id,
            "CustomerId" => $customerId
        );
        
    }
    
     public function instantBuyTransaction(InstantBuyTransaction $instantBuyTransaction) {
        $orderController = $this->apiClient->getOrders();
        
        $request = new \MundiAPILib\Models\CreateOrderRequest();
        $request->items = [new \MundiAPILib\Models\CreateOrderItemRequest()];
        $request->items[0]->description = $instantBuyTransaction->getItems()[0]->getTitle();
        $request->items[0]->quantity = $instantBuyTransaction->getItems()[0]->getQuantity();
        $request->items[0]->amount = $instantBuyTransaction->getItems()[0]->getUnitPrice();
        
        $request->customer = $this->configCustomer($instantBuyTransaction->getCustomer());
        
        $address = new \MundiAPILib\Models\CreateAddressRequest();
        $address->line1 = $instantBuyTransaction->getCustomer()->getAddress()->getNumber().', '.$instantBuyTransaction->getCustomer()->getAddress()->getStreet().', '.$instantBuyTransaction->getCustomer()->getAddress()->getNeighborhood();
        $address->line2 = $instantBuyTransaction->getCustomer()->getAddress()->getComplement();
        $address->zipCode = $instantBuyTransaction->getCustomer()->getAddress()->getZipCode();
        $address->city = $instantBuyTransaction->getCustomer()->getAddress()->getCity();
        $address->state = $instantBuyTransaction->getCustomer()->getAddress()->getState();
        $address->country = $instantBuyTransaction->getCustomer()->getAddress()->getCountry();
        
        $request->shipping = new \MundiAPILib\Models\CreateShippingRequest();
        $request->shipping->amount = "";
        $request->shipping->description = $instantBuyTransaction->getItems()[0]->getTitle();
        $request->shipping->recipientName = $instantBuyTransaction->getCustomer()->getName();
        $request->shipping->recipientPhone = "";
        $request->shipping->addressId = "";
        $request->shipping->address = $address;
        $request->shipping->type = "";
        
        $payments = [new \MundiAPILib\Models\CreatePaymentRequest()];
        $payments[0]->paymentMethod = "credit_card";
        $payments[0]->creditCard = new \MundiAPILib\Models\CreateCreditCardPaymentRequest();
        $payments[0]->creditCard->capture = false;
        $payments[0]->creditCard->installments = $instantBuyTransaction->getNrInstallment();
        $payments[0]->creditCard->cardId = $instantBuyTransaction->getCreditCardToken();
        $request->payments = $payments;   
        
        $result = $orderController->createOrder($request);
        
        if($result->charges[0]->status == "failed" || $result->charges[0]->status == "processing") {
             throw Exception::notAuthorizedMundipagg("Transação não autorizada!");
        } else {
            return array(
                "nsu" => $result->id,
                "paymentId" => $result->charges[0]->id,
                "captureUrl" => $result->charges[0]->code
            );
        }
     }
     
    /**
     * @param NonInstantBuyTransaction $nonInstantBuyTransaction
     *
     * @return string
     */
    public function nonInstantBuyTransaction(NonInstantBuyTransaction $nonInstantBuyTransaction){
        $orderController = $this->apiClient->getOrders();

        $creditCard = new \MundiAPILib\Models\CreateCreditCardPaymentRequest();
        $creditCard->capture = false;
        $creditCard->installments = $nonInstantBuyTransaction->getNrInstallment();
        $creditCard->card = new \MundiAPILib\Models\CreateCardRequest();
        $creditCard->card->number = $nonInstantBuyTransaction->getCreditCard()->getNumbers();
        $creditCard->card->holderName = $nonInstantBuyTransaction->getCreditCard()->getHolderName();
        $creditCard->card->expMonth = $nonInstantBuyTransaction->getCreditCard()->getExpDateMonth();
        $creditCard->card->expYear = $nonInstantBuyTransaction->getCreditCard()->getExpDateYear();
        $creditCard->card->cvv = $nonInstantBuyTransaction->getCreditCard()->getCvv();
        $creditCard->card->options = $nonInstantBuyTransaction->getCreditCard()->getOptions();
        $creditCard->card->privateLabel = $nonInstantBuyTransaction->getCreditCard()->getPrivateLabel();
        $creditCard->card->type = $nonInstantBuyTransaction->getCreditCard()->getType();
        $creditCard->card->metadata = $nonInstantBuyTransaction->getCreditCard()->getMetadata();
        $creditCard->card->brand = $nonInstantBuyTransaction->getCreditCard()->getFlag();
        $creditCard->card->holderDocument = $nonInstantBuyTransaction->getCreditCard()->getHolderDocument();
        $creditCard->card->privateLabel = $nonInstantBuyTransaction->getCreditCard()->getPrivateLabel();
        
        $creditCard->billingAddress = new \MundiAPILib\Models\CreateAddressRequest();
        $creditCard->billingAddress->city = $nonInstantBuyTransaction->getCreditCard()->getBillingAddress()->getCity();
        $creditCard->billingAddress->state = $nonInstantBuyTransaction->getCreditCard()->getBillingAddress()->getState();
        $creditCard->billingAddress->country = $nonInstantBuyTransaction->getCreditCard()->getBillingAddress()->getCountry();
        $creditCard->billingAddress->zipCode = $nonInstantBuyTransaction->getCreditCard()->getBillingAddress()->getZipCode();
        $creditCard->card->billingAddressId = $nonInstantBuyTransaction->getCreditCard()->getBillingAddressId();
        
        $request = new \MundiAPILib\Models\CreateOrderRequest();
        $request->items = [new \MundiAPILib\Models\CreateOrderItemRequest()];
        $request->items[0]->description = $nonInstantBuyTransaction->getItems()[0]->getTitle();
        $request->items[0]->quantity = $nonInstantBuyTransaction->getItems()[0]->getQuantity();
        $request->items[0]->amount = $nonInstantBuyTransaction->getItems()[0]->getUnitPrice();
        
        $request->customer = $this->configCustomer($nonInstantBuyTransaction->getCustomer());
        
        $address = new \MundiAPILib\Models\CreateAddressRequest();
        $address->line1 = $nonInstantBuyTransaction->getCustomer()->getAddress()->getNumber().', '.$nonInstantBuyTransaction->getCustomer()->getAddress()->getStreet().', '.$nonInstantBuyTransaction->getCustomer()->getAddress()->getNeighborhood();
        $address->line2 = $nonInstantBuyTransaction->getCustomer()->getAddress()->getComplement();
        $address->zipCode = $nonInstantBuyTransaction->getCustomer()->getAddress()->getZipCode();
        $address->city = $nonInstantBuyTransaction->getCustomer()->getAddress()->getCity();
        $address->state = $nonInstantBuyTransaction->getCustomer()->getAddress()->getState();
        $address->country = $nonInstantBuyTransaction->getCustomer()->getAddress()->getCountry();
        
        $request->shipping = new \MundiAPILib\Models\CreateShippingRequest();
        $request->shipping->amount = "";
        $request->shipping->description = $nonInstantBuyTransaction->getItems()[0]->getTitle();
        $request->shipping->recipientName = $nonInstantBuyTransaction->getCustomer()->getName();
        $request->shipping->recipientPhone = "";
        $request->shipping->addressId = "";
        $request->shipping->address = $address;
        $request->shipping->type = "";
        
        $payments = [new \MundiAPILib\Models\CreatePaymentRequest()];
        $payments[0]->paymentMethod = "credit_card";
        $payments[0]->creditCard = $creditCard;
        $request->payments = $payments;
        
        $result = $orderController->createOrder($request);
        
        if($result->charges[0]->status == "failed" || $result->charges[0]->status == "processing") {
             throw Exception::notAuthorizedMundipagg("Transação não autorizada!");
        } else {
            return array(
                "nsu" => $result->id,
                "paymentId" => $result->charges[0]->id,
                "captureUrl" => $result->charges[0]->code
            );
        }
    }
    
    public function captureTransaction($url, $paymentId){
        $chargesController = $this->apiClient->getCharges();

        $chargeId = $paymentId;
        $request = new \MundiAPILib\Models\CreateCaptureChargeRequest();
        $request->code = $url;
        
        $result = $chargesController->captureCharge($chargeId, $request);
        return $result;
    }
    
    /**
     * @param Revoke $revoke
     *
     * @return string
     */
    public function revoke(Revoke $revoke) {
        $request = array();
        
        if($revoke->getValueToCancel() !== NULL) {
            $request["amount"] = $revoke->getValueToCancel();
        }
        
        $url = $this->decodeBaseURL($revoke->getBaseURL()) . '/' . $revoke->getId();
        $responseData = $this->createRequest($url, self::DELETE, $request);
        return $responseData["id"];
    }
    
    /**
     * @param Consult $consult
     *
     * @return array
     */
    public function consult(Consult $consult) {
        $url = $this->decodeBaseURL($consult->getBaseUrl()) . '/' . $consult->getId();
        $responseData = $this->createGetRequest($url);
        return $responseData;
    }
    
    /**
     * @param String $merchantKey
     *
     * @return PaymentImpl
     */
    public static function factory($merchantKey, $basicAuthPassword) {
        return new self($merchantKey, new HttpRequest, $basicAuthPassword);
    }
    
}




