<?php

namespace Zeedhi\PaymentPlatform\examples;

$ds = DIRECTORY_SEPARATOR;

require_once __DIR__.$ds.'..'.$ds.'..'.$ds.'vendor'.$ds.'autoload.php';

use Zeedhi\PaymentPlatform\Address;

use Zeedhi\PaymentPlatform\Implementations\MundiPagg\PaymentImpl;
$paymentPlatform = PaymentImpl::factory("sk_test_kVe4L0yCEcEW4Nb9");


use Zeedhi\PaymentPlatform\BankSlip\BankSlipPlatform;
use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;

use Zeedhi\PaymentPlatform\Customer;

class Order {

    private $paymentPlatform;

    public function __construct(BankSlipPlatform $paymentPlatform) {
        $this->paymentPlatform = $paymentPlatform;
    }

    public function createOrder($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBoletoExpirationDate, Customer $customer){
        $transaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBoletoExpirationDate, $customer);
        return $this->paymentPlatform->bankSlipTransaction($transaction);
    }
}

/**
 * BankSlipTransaction
 * Os identificadores dos bancos são:
 * Banco do Brasil - BankSlipTransaction::BANCO_DO_BRASIL
 * Santander - BankSlipTransaction::SANTANDER
 * Bradesco - BankSlipTransaction::BRADESCO
 * Itau - BankSlipTransaction::ITAU
 * HSBC - BankSlipTransaction::HSBC
 * 
 * O campo 'documentNumber' deve ser informado (qualquer valor) mas não possui importância.
 * 
 * É obrigatório pelo menos 'name' em Customer.
 * Caso o campo 'type' seja setado é necessário informar o tipo de pessoa. Valores disponíveis: Customer::INDIVIDUAL or Customer::COMPANY.
 * Para a MundiPagg o campo 'state' deve conter no máximo 2 caracteres.
 * Caso não exista Customer criado basta setar os valores como o exemplo abaixo.
 * Para ter acesso ao CustomerId criado consulte a transação na class Consult.
 * Se já existe Customer criado basta setar o id($customer->setId('id'))
 */

$customer = new Customer();
$customer->setName("Stephen Vincent Strange");
$customer->setEmail("dr-stranger@avengers.com");
$customer->setDocument("666999666");
$customer->setType(Customer::INDIVIDUAL);
$customer->setPhone("999645466");
$street = "Rua Napolitano";
$number = "456";
$complement = "Casa";
$neighborhood = "Estadual";
$city = "Rio Barbosa";
$state = "PI";
$zipCode = "123123123";
$country = "BR";
$address = new Address($street, $number, $complement, $neighborhood, $city, $state, $zipCode, $country);
$customer->setAddress($address);

$bank = BankSlipTransaction::BANCO_DO_BRASIL;
$documentNumber = "123458852";
$instructions = "Essas são as palavras que já foram ditas...";
$daysToAddInBoletoExpirationDate = 5;

$value = "1500.00";
$transactionId = "0666";

$order = new Order($paymentPlatform);
$transactionKey = $order->createOrder($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBoletoExpirationDate, $customer);
var_dump($transactionKey);
