module.exports = function(grunt) {
    var fs = require('fs');
    var clientConfig = JSON.parse(fs.readFileSync('config.json', 'utf8')).client;
    var cacheBypassConfig = 'CacheBypassConfig.js';
    var repositoryPath = 'js/model/**/*.js';
    var servicePath    = 'js/services/*.js';
    var controllerPath = 'js/controllers/*.js';

    var assetsPath = [
        cacheBypassConfig,
        repositoryPath,
        servicePath,
        controllerPath
    ];

    grunt.initConfig({
        concat: {
            none: {},
            zh: {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '// FILE: ' + filepath + '\n' + src;
                    }
                },
                src: assetsPath,
                dest: 'dist/nocash.js'
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                // the banner is inserted at the top of the output
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'dist/mainmtc.min.js': assetsPath
                }
            }
        },
        concat_css: {
            options: {},
            all: {
                src: [
                    'css/style.css', 
                    'css/'+clientConfig+'.css'
                ],
                dest: "dist/main.css"
            }
        },
        watch: {
            pivotal: {
                files: assetsPath,
                tasks: ['concat', 'cachebreaker']
            }
        },
        zhIdGenerator: {
            options: {
                jsonPath: ["../mobile/json"],
                replaceIds: true
            }
        },
        cachebreaker: {
            dev: {
                options: {
                    match: ['nocash.js']
                },
                files: {
                    src: ['index.html']
                }
            }
        }
    });

    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-cache-breaker');
    grunt.loadNpmTasks('grunt-concat-css');

    // Default task.
    grunt.registerTask('default', ['concat_css', 'concat']);
};