module.exports = function(grunt) {
	var srcPaths = {
		js: [
			'src/controller/*.js'
		],
		css: [
			'src/style/*.css'
		],
		html: [
			'src/template/*.html'
		]
	};

    grunt.initConfig({
        concat: {
            none: {},
            zh: {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '// FILE: ' + filepath + '\n' + src;
                    }
                },
                src: srcPaths.js,
                dest: 'dist/es6/odh-radio-es6.js'
            },
            css: {
                src: srcPaths.css,
                dest: 'dist/odh-radio.css'
            }
        },
        jshint: {
            all: ['Gruntfile.js', srcPaths.js],
            options: {
            	esversion: 6
            }
        },
        watch: {
            js: {
				files: srcPaths.js,
				tasks: ['concat', 'babel']
			},
			html: {
				files: srcPaths.html,
				tasks: ['copy']
			},
			css: {
				files: srcPaths.css,
				tasks: ['concat']
			}
        },
		babel: {
			options: {
				sourceMap: true,
				presets: ['es2015']
			},
			dist: {
				files: {
					'dist/odh-radio.js': 'dist/es6/odh-radio-es6.js'
				}
			}
		},
		copy: {
			main: {
				files: [
					{
						expand: true,
						cwd: 'src/',
						src: 'template/*',
						dest: 'dist/'
					}
				]
			}
		}
    });

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-copy');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-babel');

    // Default task.
    grunt.registerTask('default', ['concat', 'babel', 'copy']);
};