describe("DataSource", function () {
	/*jslint evil: true */
	var eventEngine = new EventEngine(
		new EventFactory(new MethodBuilder({}, {
			$eval: function (a, b) {
				a = "var args = b.args;" + a;
				return eval(a, b);
			}
		}), {}),
		new EventHandler({}, {}, console),
		{
			updateTemplate: function () {
				updateTemplate = true;
			}
		},
		{},
		{ logEvent: function () { } }
	);
	var dataSource = new DataSource(null, {}, eventEngine);
	it("Must add a function as a listener when passed by params", function () {
		var wordTest = "fail";
		dataSource.afterLoadDataSource(function () {
			wordTest = "OK";
		});
		dataSource.afterLoadDataSource();
		expect(wordTest).toEqual("OK");
	});
	it("Method select all should mark all items as __selected", function () {
		var dataSource = new DataSource(null, {}, eventEngine);
		dataSource.data = [{}, {}, {}, {}];
		dataSource.selectAll();
		dataSource.data.forEach(function (item) {
			expect(item.__isSelected).toEqual(true);
		});
		expect(dataSource.checkedRows.length).toEqual(4);
	});
	it("Method unSelectAll should mark all items as not __selected", function () {
		var dataSource = new DataSource(null, {}, eventEngine);
		dataSource.data = [{}, {}, {}, {}];
		dataSource.unSelectAll();
		dataSource.data.forEach(function (item) {
			expect(item.__isSelected).toEqual(false);
		});
		expect(dataSource.checkedRows.length).toEqual(0);
	});

	it("Method remove: should only return a promise if ", function (done) {
		var zhPromise = {
			when: function () { return new ZHPromise(Q).when(); }
		};
		spyOn(zhPromise, 'when').and.callThrough();
		var dataSource = new DataSource(null, {}, eventEngine, {}, zhPromise);
		dataSource.remove().then(function () {
			expect(zhPromise.when).toHaveBeenCalled();
			done();
		});
	});

	it("Method remove: call removeCheckedRows and return the storage strategy result", function (done) {
		var zhPromise = new ZHPromise(Q);
		var storageStrategy = {
			remove: function () { return zhPromise.when("myReturn"); }
		};
		var dataSource = new DataSource(storageStrategy, {}, eventEngine, {}, zhPromise);
		spyOn(dataSource, "removeCheckedRows").and.callThrough();
		spyOn(storageStrategy, "remove").and.callThrough();

		dataSource.remove().then(function (result) {
			expect(dataSource.removeCheckedRows).toHaveBeenCalled();
			expect(storageStrategy.remove).toHaveBeenCalled();
			expect(result).toEqual("myReturn");
			done();
		});
	});

	it("Method getNextpage: should return the currentPage plus one", function () {
		var dataSource = new DataSource(null, {}, eventEngine, {});
		expect(dataSource.getNextPage()).toEqual(2);
	});

	it("Method getPrevPage: should return the currentPage plus one", function () {
		var dataSource = new DataSource(null, {}, eventEngine, {});
		expect(dataSource.getPrevPage()).toEqual(0);
	});

	it("Method getLastRow: should return the last data element", function () {
		var dataSource = new DataSource(null, {}, eventEngine, {});
		dataSource.data = [10, 20, 30, 40];
		expect(dataSource.getLastRow()).toEqual(40);
	});

	it("Method newRow: should return the new length of the dataSource data", function () {
		var dataSource = new DataSource(null, {}, eventEngine, {});
		dataSource.data = null;
		expect(dataSource.newRow()).toEqual(1);
	});

	it("Method setData: should set the data as the argument if the currentPage is 1, and fire afterLoadDataSource", function () {
		var dataSource = new DataSource(null, {}, eventEngine, {});
		dataSource.data = null;
		dataSource.afterLoadDataSource = function () { };
		spyOn(dataSource, "afterLoadDataSource").and.callThrough();
		var data = [{}];
		dataSource.setData(data);
		expect(dataSource.afterLoadDataSource).toHaveBeenCalled();
		expect(dataSource.data).toEqual(data);
	});

	it("Method setData: should concat the data to the argument if the currentPage is different than 1, and fire afterLoadDataSource", function () {
		var dataSource = new DataSource(null, {}, eventEngine, {});
		dataSource.data = null;
		dataSource.afterLoadDataSource = function () { };
		spyOn(dataSource, "afterLoadDataSource").and.callThrough();
		var data = [{}];
		dataSource.setData(data);
		expect(dataSource.afterLoadDataSource).toHaveBeenCalled();
		expect(dataSource.data).toEqual(data);

		dataSource.getNextPage();
		dataSource.setData(data);
		expect(dataSource.data.length).toBe(2);
		expect(dataSource.afterLoadDataSource).toHaveBeenCalled();
	});

	it("Method load: should call the storageStrategy getDataSource and return its result", function (done) {
		var zhPromise = new ZHPromise(Q);
		var getDataSourceResult = [{ someItem: "value" }];
		var storageStrategy = {
			getDataSource: function (name, callback) {
				callback(getDataSourceResult);
				return zhPromise.when(getDataSourceResult);
			}
		};
		var templateManager = {
			updateTemplate: function () { }
		};
		var dataSource = new DataSource(storageStrategy, templateManager, eventEngine, {});
		var callbackMock = {
			callback: function () { }
		};
		spyOn(callbackMock, "callback").and.callThrough();
		spyOn(dataSource, "setData").and.callThrough();
		dataSource.load(null, null, null, callbackMock.callback)
			.then(function (result) {
				expect(callbackMock.callback).toHaveBeenCalled();
				expect(dataSource.setData).toHaveBeenCalled();
				expect(result).toEqual(getDataSourceResult);
				expect(dataSource.data).toEqual(getDataSourceResult);
				done();
			});
	});

	it("Method load: should fire afterLoadDataSourceError if there was an error getting the dataSource from the storageStrategy", function (done) {
		var zhPromise = new ZHPromise(Q);
		var storageStrategy = {
			getDataSource: function () {
				return zhPromise.reject();
			}
		};
		var templateManager = {
			updateTemplate: function () { }
		};
		var dataSource = new DataSource(storageStrategy, templateManager, eventEngine, {});

		spyOn(dataSource, "afterLoadDataSourceError").and.callThrough();
		dataSource.load(null, null, null)
			.then(function (result) {
				expect(dataSource.afterLoadDataSourceError).toHaveBeenCalled();
				done();
			});
	});

	it("Method reload: should call dataSource.load", function () {
		var dataSource = new DataSource(null, {}, eventEngine, {});
		dataSource.load = function () { };
		spyOn(dataSource, "load").and.callThrough();
		dataSource.reload();
		expect(dataSource.load).toHaveBeenCalled();
	});

	it("Method filter: should only call zhPromise.when if there is no storage strategy", function () {
		var zhPromise = {
			when: function () { return new ZHPromise(Q).when(); }
		};
		spyOn(zhPromise, 'when').and.callThrough();
		var dataSource = new DataSource(null, {}, eventEngine, {}, zhPromise);
		dataSource.filter();
		expect(zhPromise.when).toHaveBeenCalled();
	});

	it("Method filter: should return the dataSet data based on the dataSource name", function (done) {
		var zhPromise = new ZHPromise(Q);
		var getDataSourceResultData = [{}];
		var getDataSourceResult = {
			dataset: {
				WOWSUCHDATASOURCE: getDataSourceResultData
			}
		};
		var storageStrategy = {
			getDataSource: function () {
				return zhPromise.when(getDataSourceResult);
			}
		};
		var dataSource = new DataSource(storageStrategy, {}, eventEngine, new DataSourceHolder());
		dataSource.name = "WOWSUCHDATASOURCE";
		dataSource.filter()
			.then(function (data) {
				expect(data).toEqual(getDataSourceResultData);
				done();
			});
	});

	it("Method filter: should only return the storageStrategy result if there is no dataset with the datasource name", function (done) {
		var zhPromise = new ZHPromise(Q);
		var getDataSourceResultData = [{}];
		var getDataSourceResult = {
			DATASEUTDIFFERENT: {
				WOWSUCHDATASOURCE: getDataSourceResultData
			}
		};
		var storageStrategy = {
			getDataSource: function (name, callback) {
				return zhPromise.when(getDataSourceResult);
			}
		};
		var dataSource = new DataSource(storageStrategy, {}, eventEngine, {}, new DataSourceHolder());
		dataSource.name = "WOWSUCHDATASOURCE";
		dataSource.filter()
			.then(function (data) {
				expect(data).toEqual(getDataSourceResult);
				done();
			});
	});

	it('Timeout should be project default if not specified', function () {
		var DATA_SOURCE_NAME = 'dataSourceName',
			PROJECT_TIMEOUT = 12345;

		var zhPromise = new ZHPromise(Q),
			templateManager = {
				project: {
					timeout: PROJECT_TIMEOUT
				}
			},
			requestEngine = {
				doRequest: function () { return { then: function () { } }; }
			},
			requestFactory = {
				factory: function () {
					return arguments[0];
				}
			},
			methodBuilder = {},
			screenService = {},
			eventEngine = { createEvent: function () { } },
			dataSourceHolder = { getDataSource: function () { } };

		var storageStrategy = new RequestDataSourceEngine(requestEngine, requestFactory, methodBuilder, screenService, zhPromise, ZhUtil, dataSourceHolder, templateManager);

		spyOn(storageStrategy, 'getDataSource').and.callThrough();
		spyOn(requestEngine, 'doRequest').and.callThrough();

		var dataSource = new DataSource(storageStrategy, templateManager, eventEngine, dataSourceHolder, zhPromise);
		dataSource.name = DATA_SOURCE_NAME;
		dataSource.load();
		expect(storageStrategy.getDataSource).toHaveBeenCalledTimes(1);
		expect(storageStrategy.getDataSource).toHaveBeenCalledWith(DATA_SOURCE_NAME, jasmine.any(Function), jasmine.any(Object), jasmine.any(Number), jasmine.any(Number), undefined, jasmine.any(Number), undefined, undefined);
		expect(requestEngine.doRequest).toHaveBeenCalledTimes(1);
		expect(requestEngine.doRequest).toHaveBeenCalledWith(jasmine.objectContaining({
			'timeout': PROJECT_TIMEOUT
		}));
	});
});