function LayoutPreferences(templateManager, ScreenService, Query, ZHPromise, carouselWidgetCreator, eventAggregator, zhCustomizationsService, eventEngine, containerMerger, preferencesUserService) {

	var self = this;
	var protectedMethods = {};

	var widget, preferenceActivated;

	var fieldDefaultProperties = {
		isVisible: true,
		showOnList: true,
		visibility: true,
		grouped: false,
		fixed: false
	};

	eventAggregator.subscribe("OnListStartUp", getAndApplyDefaultPreference);

	function getAndApplyDefaultPreference(eventData) {
		if (eventData.widget) {
			var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
			self.getDefaultPreferenceByUser(eventData.widget, userRegistered).then(function(defaultPreference) {
				if (defaultPreference) {
					self.applyLayout(defaultPreference, eventData.widget);
				} else {
					eventData.widget.customizationsApplied = eventData.widget.customizationsApplied || {};
					eventData.widget.customizationsApplied.layout = "default";
				}
			});
		}
	}

	this.getDefaultPreferenceByUser = function(widget, userId) {
		return zhCustomizationsService.getWidgetCustomizations(widget, "layout").then(function(preferences) {
			return preferences.filter(function(preference) {
				return preference.mapUserIdDefault[userId];
			}).shift();
		});
	};

	this.startConfig = function(editingWidget) {
		widget = editingWidget;
		ScreenService.openPopup(carouselWidgetCreator.createCaroulselWidget(editingWidget, functionsAsArguments));
	};

	this.restoreDefault = function(editingWidget) {
		self.restoreDefaultLayout(editingWidget);
	};

	this.preferencesListOnEnter = function(widgetPreferenceList) {
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		return zhCustomizationsService.getWidgetCustomizations(widget, "layout").then(function(preferences) {
			preferences.forEach(function(preference) {
				preference.defaultPreference = preference.mapUserIdDefault[userRegistered] || false;
				preference.preferenceTypeLabel = zhCustomizationsService.getLabelRepositoryType(preference.customizationRepository);
			});
			widgetPreferenceList.dataSource.data = preferences;
		});
	};

	this.applyLayout = function(layout, widget) {
		if (widget.backupPDFOrientation === undefined) {
			widget.backupPDFOrientation = widget.reportOrientation || 'portrait';
		}

		widget.reportOrientation = layout.pdfReportOrientation || widget.backupPDFOrientation;
		containerMerger.merge(templateManager.container, layout.containerDiff, "original");
		if (!widget.customizationsApplied) {
			widget.customizationsApplied = {};
		}
		widget.customizationsApplied.layout = layout._id;
		redrawWidget(widget);
	};


	function applyLayoutEventCallback(eventData) {
		var customization = eventData.e.customization;
		var editingWidget = eventData.e.editingWidget;
		self.restoreDefaultLayout(editingWidget);
		if (!customization.isDefaultOption) {
			self.applyLayout(customization, editingWidget);
		}
	}

	this.restoreDefaultLayout = function(widget) {
		widget.reportOrientation = widget.backupPDFOrientation || 'portrait';
		widget.customizationsApplied = widget.customizationsApplied || {};
		widget.customizationsApplied.layout = "default";

		containerMerger.restore(templateManager.container, "original");
		redrawWidget(widget);
	};

	function redrawWidget(widget) {
		if (widget.redraw) {
			widget.redraw(true);
		}
		templateManager.updateTemplate();
	}

	eventEngine.bindEvent(zhCustomizationsService, "LayoutPreferenceOnClick", applyLayoutEventCallback);

	var openNewPreferenceWidget = function(widget) {
		var caroulsel = widget.parentWidget;
		var popupAddPreference = caroulsel.widgets[3];
		var typeField = popupAddPreference.getField("type");
		var allowedPreferences = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		typeField.dataSource.data = allowedPreferences;
		popupAddPreference.newRow();
		ScreenService.openPopup(popupAddPreference);
	};

	var openPopupSelectDestinyRepository = function(preferencesListWidget) {
		var changePreferenceTypeWidget = preferencesListWidget.parentWidget.widgets[4];
		changePreferenceTypeWidget.newRow();
		var allowedPreferences = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		changePreferenceTypeWidget.getField("preferenceType").dataSource.data = allowedPreferences;
		var toTransferLayouts = preferencesListWidget.getCheckedRows();
		var isSomeLayoutConnectedToAnVisuzalization = false;
		var promises = toTransferLayouts.map(function(layout) {
			return zhCustomizationsService.getCustomizationView(widget, layout.customizationRepository, layout.customizationType, layout._id).then(function(connectedView) {
				if (connectedView) {
					isSomeLayoutConnectedToAnVisuzalization = true;
				}
			});
		});
		ZHPromise.all(promises).then(function() {
			if (!isSomeLayoutConnectedToAnVisuzalization) {
				ScreenService.openPopup(changePreferenceTypeWidget);
			} else {
				ScreenService.showMessage(ScreenService.i18n("This layout is connected with a customizations. It can't be transferred."));
			}
		});
	};

	var saveTransferLayoutClick = function(widgetChangePreferenceType) {
		var preferencesListWidget = widgetChangePreferenceType.parentWidget.widgets[0];
		var toChangePreferences = preferencesListWidget.getCheckedRows();
		var newLayoutType = widgetChangePreferenceType.getField("preferenceType").value();
		zhCustomizationsService.transferCustomizationsRepository(widget, toChangePreferences, newLayoutType).then(function() {
			self.preferencesListOnEnter(preferencesListWidget);
			ScreenService.closePopup();
		});
	};

	var applyLayoutActionClick = function(row) {
		self.applyLayout(row, widget);
		ScreenService.closePopup(true);
		widget.activate();
	};

	var closePopupAndRunDigest = function() {
		ScreenService.closePopup();
		widget.activate();
		ScreenService.updateTemplate();
	};

	var createNewPreference = function(widget) {
		if (widget.isValid()) {
			var carousel = widget.parentWidget;
			var preferencesListWidget = carousel.widgets[0];
			var preferenceName = widget.currentRow.name;
			var customizationRepository = widget.currentRow.type;
			var pdfReportOrientation = widget.currentRow.pdfReportOrientation;
			var preferenceHeader = createPreferenceHeader(preferenceName, customizationRepository, pdfReportOrientation);

			ScreenService.closePopup();
			templateManager.updateTemplate();
			self.preferencesListOnEnter(preferencesListWidget).then(function() {
				openEditPreference(preferencesListWidget, preferenceHeader);
			});
		}
	};

	var openEditPreference = function(widgetBefore, row) {
		activatePreference(row);
		var indexNextWidget = widgetBefore.parentWidget.getNextIndex();
		var nextWidget = widgetBefore.parentWidget.widgets[indexNextWidget];
		nextWidget.dataSource.data = getFieldEditingWidgetAsDataSource(widget);
		var widgetDifferences = getEditingWidgetPath(row.containerDiff.widgets);
		nextWidget.propertiesToChange = {};
		nextWidget.label = row.label;
		nextWidget.dataSource.data = sortFieldsOrder(nextWidget.dataSource.data.map(function(field, index) {
			delete field.__isSelected;
			delete field.__leftActions;
			delete field.__notSelectable;
			delete field.__rightActions;
			if (!field.sequenceInList) {
				field.sequenceInList = index + 1;
			}
			field.qtdRows = nextWidget.dataSource.data.length;
			if (!Util.isEmptyOrBlank(widgetDifferences.fields)) {
				var fieldPreference = filterObject(widgetDifferences.fields, field.id, "id");
				if (fieldPreference.obj && Object.keys(fieldPreference.obj).length) {
					return Util.extend(field, fieldPreference.obj, true);
				}
			}
			return Util.extend(fieldDefaultProperties, field, true);
		}));
		widgetBefore.parentWidget.nextWidget();
	};

	function getFieldEditingWidgetAsDataSource(gridWidget) {
		return gridWidget.fields.map(function(field) {
			var data = {
				"id": field.id,
				"name": field.name,
				"label": field.label,
				"align": field.align,
				"isVisible": field.isVisible,
				"showOnList": field.showOnList,
				"isVisibleOnReport": field.isVisible && field.showOnList,
				"visibility": field.isVisible && field.showOnList,
				"grouped": field.grouped || false,
				"fixed": field.fixed || false,
				"skipPreference": field.skipPreference || false
			};
			return data;
		}).filter(function(field) {
			return !field.skipPreference;
		});
	}

	var changeOrder = function(operator, widget) {
		var lastUnchecked;
		if (operator === 1) {
			widget.dataSource.data.reverse();
		}

		widget.dataSource.data.forEach(function(row, index) {
			if (row.__isSelected) {
				if (lastUnchecked && ((row.sequenceInList > 1 && operator == -1) || (row.sequenceInList < widget.dataSource.data.length && operator == 1))) {
					row.sequenceInList += operator;
					if (lastUnchecked.sequenceInList == row.sequenceInList) {
						lastUnchecked.sequenceInList -= operator;
					}
				}
			} else {
				lastUnchecked = row;
			}
		});
		widget.dataSource.data = sortFieldsOrder(widget.dataSource.data);
	};

	var sortFieldsOrder = function(nextWidgetDataSource) {
		return nextWidgetDataSource.sort(function(firstField, secondField) {
			return firstField.sequenceInList - secondField.sequenceInList;
		});
	};

	var activatePreference = function(preference) {
		preferenceActivated = preference;
	};

	var changePreferenceVisibility = function(row, widget) {
		row.isVisible = true;
		row.showOnList = row.isVisibleOnReport = row.visibility;
	};

	var setAsDefaultPreference = function(row, widget) {
		var toUpdate = [];
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		var preferenceSetAsDefault = widget.dataSource.data.filter(function(preference) {
			return preference.mapUserIdDefault[userRegistered] === true && preference._id !== row._id;
		}).shift();
		if (preferenceSetAsDefault) {
			preferenceSetAsDefault.mapUserIdDefault[userRegistered] = false;
			removeNotInterestingProperties(preferenceSetAsDefault);
			toUpdate.push(preferenceSetAsDefault);
		}
		removeNotInterestingProperties(row);
		row.mapUserIdDefault[userRegistered] = !row.mapUserIdDefault[userRegistered];
		toUpdate.push(row);
		zhCustomizationsService.saveCustomization(toUpdate, 'layout').then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				ScreenService.hideLoader();
				ScreenService.updateTemplate();
			});
		});
	};

	var deletePreferenceClick = function(widget) {
		return zhCustomizationsService.removeCustomizations(widget.getCheckedRows()).then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				templateManager.updateTemplate();
			});
		});
	};

	var createPreferenceHeader = function(preferenceName, customizationRepository, pdfReportOrientation) {
		var header = {
			"label": preferenceName,
			"widgetId": widget.id,
			"containerId": widget.container.id,
			"customizationRepository": customizationRepository,
			"unsync_id": new Date().getTime(),
			"customizationType": "layout",
			"mapUserIdDefault": {
				"default": false
			},
			"defaultPreference": false,
			"pdfReportOrientation": pdfReportOrientation,
			"containerDiff": createDiffDataStructure(widget.container, widget)
		};
		return header;
	};

	var createDiffDataStructure = function(container, widget) {
		var containerDiff = {};
		containerDiff.id = container.id;
		createWidgetsDataStructure(widget.id, container.widgets, containerDiff);
		return containerDiff;
	};

	var createWidgetsDataStructure = function(widgetId, widgets, containerDiff) {
		containerDiff.widgets = [];
		createWidgetsObjects(containerDiff.widgets, widgets, widgetId);
		if (Util.isArray(widgets) && widgets.length > 0) {
			widgets.forEach(function(eachWidget) {
				createWidgetsDataStructure(widgetId, eachWidget.widgets, getWidgetObjectById(containerDiff.widgets, eachWidget.id));
			});
		}
	};

	var getWidgetObjectById = function(widgets, widgetId) {
		return widgets.filter(function(eachWidgetObject) {
			return eachWidgetObject.id == widgetId;
		}).shift();
	};

	var createWidgetsObjects = function(widgetsDiff, widgetsArray, widgetId) {
		if (Util.isArray(widgetsArray) && widgetsArray.length > 0) {
			widgetsArray.forEach(function(eachWidget) {
				var properties = {
					"id": eachWidget.id
				};
				if (eachWidget.id == widgetId) {
					properties.fields = [];

				}
				widgetsDiff.push(properties);
			});
		}
	};

	var getEditingWidgetPath = function(widgetsArray) {
		var foundWidget = getWidgetObjectById(widgetsArray, widget.id);
		if (!foundWidget) {
			widgetsArray.some(function(eachWidget) {
				foundWidget = getEditingWidgetPath(eachWidget.widgets);
				return foundWidget;
			});
		}
		return foundWidget;
	};

	var updatePreferenceData = function(fields, preferencesWidgetList) {
		var caroulselWidget = preferencesWidgetList.parentWidget;
		var editingWidget = getEditingWidgetPath(preferenceActivated.containerDiff.widgets);
		removeNotInterestingProperties(preferenceActivated);
		if (Util.isArray(fields)) {
			editingWidget.fields = fields;
		} else {
			var index = filterObject(editingWidget.fields, fields.id, "id").index;
			editingWidget.fields[index] = fields;
		}
		return zhCustomizationsService.saveCustomization(preferenceActivated, 'layout').then(function() {
			self.preferencesListOnEnter(preferencesWidgetList).then(function() {
				caroulselWidget.previousWidget();
				templateManager.updateTemplate();
			});
		});
	};

	var removeNotInterestingProperties = function(object) {
		Object.keys(object).forEach(function(key) {
			if (~key.indexOf('__')) {
				delete object[key];
			}
		});
	};

	var saveFieldSettings = function(widgetPreferenceFields) {
		ScreenService.showLoader();
		var caroulselWidget = widgetPreferenceFields.parentWidget;
		var widgetPreferenceList = caroulselWidget.widgets[0];
		var fields = widgetPreferenceFields.dataSource.data;
		return updatePreferenceData(fields, widgetPreferenceList).then(function() {
			ScreenService.hideLoader();
		});
	};

	var saveAndApplyPreference = function(widgetPreferenceFields) {
		return saveFieldSettings(widgetPreferenceFields).then(function() {
			applyLayoutActionClick(preferenceActivated);
		});
	};

	var openPopupPreferencesProperty = function(widget, row) {
		var indexNextWidget = widget.parentWidget.getNextIndex();
		var nextWidget = widget.parentWidget.widgets[indexNextWidget];
		nextWidget.currentFieldId = row.id;
		nextWidget.setCurrentRow(row);
		nextWidget.label = ScreenService.i18n("Column properties") + " - " + row.label;
		widget.parentWidget.nextWidget();
	};

	var backToPreviousWidget = function(widget) {
		widget.parentWidget.previousWidget();
	};

	var saveFieldChanges = function(widgetProperties, row) {
		var preferencesListWidget = widgetProperties.parentWidget.widgets[0];
		return updatePreferenceData(row, preferencesListWidget);
	};

	//Controller Utilities
	var filterObject = function(obj, value, key) {
		var filtered = obj.filter(function(preference) {
			return preference[key] == value;
		});
		var toReturn = {};
		toReturn.index = obj.indexOf(filtered[0]);
		toReturn.obj = filtered[0];
		return toReturn;
	};

	var functionsPreferencesListWidget = {
		"openEditPreference": openEditPreference,
		"deletePreferenceClick": deletePreferenceClick,
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"setAsDefaultPreference": setAsDefaultPreference,
		"applyLayoutActionClick": applyLayoutActionClick,
		"openNewPreferenceWidget": openNewPreferenceWidget,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository,
		"preferencesListOnEnter": self.preferencesListOnEnter
	};

	var functionsPreferenceFieldsListWidget = {
		"backToPreviousWidget": backToPreviousWidget,
		"changePreferenceVisibility": changePreferenceVisibility,
		"openPopupPreferencesProperty": openPopupPreferencesProperty,
		"saveFieldSettings": saveFieldSettings,
		"changeOrder": changeOrder,
		"saveAndApplyPreference": saveAndApplyPreference
	};

	var functionsFieldsPropertiesListWidget = {
		"backToPreviousWidget": backToPreviousWidget,
		"saveFieldChanges": saveFieldChanges
	};

	var functionsNewPreferenceWidget = {
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"createNewPreference": createNewPreference
	};

	var functionsChangePreferenceTypeWidget = {
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository,
		"saveTransferLayoutClick": saveTransferLayoutClick
	};

	var functionsTransferCustomizationRepository = {
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"saveTransferLayoutClick": saveTransferLayoutClick
	};

	var functionsAsArguments = {
		"functionsPreferencesListWidget": functionsPreferencesListWidget,
		"functionsPreferenceFieldsListWidget": functionsPreferenceFieldsListWidget,
		"functionsFieldsPropertiesListWidget": functionsFieldsPropertiesListWidget,
		"functionsNewPreferenceWidget": functionsNewPreferenceWidget,
		"functionsChangePreferenceTypeWidget": functionsChangePreferenceTypeWidget
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register("zhLayoutPreferences", LayoutPreferences);
});